<?php
namespace Teknisa\CADEST\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Model\Geral\Entities\Bairro;
use Teknisa\Libs\Util\DateUtil;

class EnderecoController extends \Teknisa\CADEST\Controllers\TeknisaCrud {

    protected $dataSourceName = "paramfilial";
	protected $crudService;
	protected $dataSourceManager;
	protected $environment;
    protected $entityManager;

	public function __construct($crudService, ParameterBag $parameterBag, $dataSourceManager, $environment, $entityManager) {
		$this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
	}

	public function validateNeighborhood(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'bairro';
        $this->parameterBagColumns = array("CDPAIS", "SGESTADO", "CDMUNICIPIO");
        
        $filter = $request->getFilterCriteria()->getConditions();
        if($filter){
            $filterToFind = array_filter($filter, function($element){
                if($element['operator'] != 'LIKE_ALL')
                    return $element; 
            });
            $neighborhoodIndex = array_filter($filter, function($element){
                if($element['operator'] == 'LIKE_ALL')
                    return $element; 
            });
            $country = null;
            $state = null;
            $county = null;
            $neighborhood = null;
            foreach($filterToFind as $adressFilter){
                switch($adressFilter['columnName']){
                    case "CDPAIS": 
                        $country = $adressFilter['value'];
                        break;
                    case "SGESTADO":
                        $state = $adressFilter['value'];
                        break;
                    case "CDMUNICIPIO": 
                        $county = $adressFilter['value'];
                        break;
                }
            }
    
            foreach($neighborhoodIndex as $neighborhoodArray)
                $neighborhood = $neighborhoodArray['value'];
                
            $neighborhood = str_replace('%', '', $neighborhood);
            
            
            $result = $this->crudService->getFilteredDataSet($this->dataSourceName, $filterToFind);
            
            $neighborhoods = $result->getRows();
            
            if(!$neighborhoods)
                $sequential = '0000000001';
            else{
                
                $largest = 0;
    
                for( $i = 0; $i < count($neighborhoods); $i++){
                    if($largest < $neighborhoods[$i]['CDBAIRRO']) {
                    $largest = $neighborhoods[$i]['CDBAIRRO']; 
                    }
                }
                $sequential = $largest+1;
                $sequential = str_pad($sequential, 10, "0", STR_PAD_LEFT); 
            }
            $neighborhood = $this->insertNewNeighborhood($sequential, $country, $state, $county, $neighborhood);
        }
        
        parent::find($request, $response);
    }
    
    public function insertNewNeighborhood($sequential, $country, $state, $county, $neighborhood){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrgTrab();
            $bairro = new Bairro();
            $bairro->setCdbairro($sequential);
            $bairro->setCdpais($country);
            $bairro->setSgestado($state);
            $bairro->setCdmunicipio($county);
            $bairro->setNmbairro($neighborhood);
            $bairro->setNrorg($nrorg);
            $bairro->setDtinclusao(DateUtil::getDataAtual());
            $this->entityManager->persist($bairro);
            $this->entityManager->flush();
            $connection->commit();
            return $bairro;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }


    public function alterarEstrutura(DTO\Request $request, DTO\Response $response) {
        try {
            $operador = $request->getRow();
            $this->alterarEstruturaService($operador['CDFILIAL'], $this->environment->getCdoperador(), $operador['NRORG']);
            $row = array();
            $row['CDFILIAL'] = $operador['CDFILIAL'];
            $row['CDOPERADOR'] = $this->environment->getCdoperador();
            $row['NRORG'] = $operador['NRORG'];
            $dataSet = new DataSet('operador', array($row));
            $response->setDataSets(array($dataSet));
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    public function alterarEstruturaService($cdfilial, $cdoperador, $nrorg) {
        $operador = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $cdoperador,
                    'nrorg' => $nrorg
                )
            );
        $operador->setNrorgtrab($nrorg);
        $operador->setCdfilialselecionada($cdfilial);
        $operador->setNrorgselecionada($nrorg);
        $this->entityManager->persist($operador);
        $this->entityManager->flush();
        $this->environment->setCdFilial($cdfilial);
    }
}
